﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using ELTE.Windows.CoursesListInModelView.Model;
using System.Windows.Input;
using System.Windows.Data;
using System.Globalization;

namespace ELTE.Windows.CoursesListInModelView.ViewModel
{
    /// <summary>
    /// Kurzusok nézetmodell típusa.
    /// </summary>
    public class CoursesViewModel : ViewModelBase
    {
        private readonly UniversityEntities _UniversityEntities; // adatforrás
        private RelayCommand _LoadCommand; // betöltés parancsa
        private RelayCommand _ExitCommand; // kilépés parancsa
        private RelayCommand _SortCommand; // rendezés parancsa
        private ObservableCollection<Course> _CourseList; // kurzusok listája
        private ICollectionView _CoursesView; // kurzusok nézete (a szűréshez)
        private ListSortDirection _LastSortDirection = ListSortDirection.Ascending; // rendezési mód

        /// <summary>
        /// Betöltés parancs lekérdezése.
        /// </summary>
        public ICommand LoadCommand
        {
            get
            {
                if (_LoadCommand == null) // ha még nem létezik
                {
                    _LoadCommand = new RelayCommand(param => CanLoad(), param => Load());
                    // létrehozzuk a parancsot úgy, hogy a megadott metódusokat használja fel
                }
                return _LoadCommand;
            }
        }

        /// <summary>
        /// Kilépés parancs lekérdezése.
        /// </summary>
        public ICommand ExitCommand
        {
            get
            {
                if (_ExitCommand == null)
                    _ExitCommand = new RelayCommand(param => Exit());
                return _ExitCommand;
            }
        }

        /// <summary>
        /// Rendezés parancs lekérdezése.
        /// </summary>
        public ICommand SortCommand
        {
            get
            {
                if (_SortCommand == null)
                    _SortCommand = new RelayCommand(param => Sort());
                return _SortCommand;
            }
        }

        /// <summary>
        /// Kurzusok listájának lekrédezése, vagy beállítása.
        /// </summary>
        public ObservableCollection<Course> CourseList
        {
            get { return _CourseList; }
            set
            {
                _CourseList = value;
                OnPropertyChanged("CourseList");
            }
        }

        /// <summary>
        /// Akutálisan kiválasztott kurzus lekérdezése.
        /// </summary>
        public Course CurrentCourse
        {
            get
            {
                if (_CoursesView != null)
                    return _CoursesView.CurrentItem as Course;
                else
                    return null;
            }
        }

        /// <summary>
        /// Keresendő kurzus beállítása.
        /// </summary>
        public string CourseToLookFor
        {
            set
            {
                _CoursesView.Filter = item =>
                {
                    return item is Course
                        && (item as Course).Name.StartsWith(value, true, CultureInfo.CurrentUICulture);
                };
            }
        }

        public CoursesViewModel()
        {
            _UniversityEntities = new UniversityEntities();
        }


        /// <summary>
        /// Megadja, lehetséges-e a betöltés.
        /// </summary>
        /// <returns></returns>
        private Boolean CanLoad()
        {
            return (!_UniversityEntities.Course.First().Application.IsLoaded);
            // akkor lehetséges, ha még nincs betöltve
        }

        /// <summary>
        /// Adatok betöltése.
        /// </summary>
        private void Load()
        {
            CourseList = new ObservableCollection<Course>(_UniversityEntities.Course.Include("Application").Include("Application.Student"));
            _CoursesView = CollectionViewSource.GetDefaultView(CourseList);

            if (_CoursesView != null)
                _CoursesView.CurrentChanged += OnCollectionViewCurrentChanged;

            OnPropertyChanged("CurrentCourse");
        }

        /// <summary>
        /// Alkalmazásból történő kilépés.
        /// </summary>
        private static void Exit()
        {
            System.Windows.Application.Current.Shutdown();
        }

        /// <summary>
        /// Adatok rendezése.
        /// </summary>
        private void Sort()
        {
            var direction = _LastSortDirection == ListSortDirection.Ascending ? ListSortDirection.Descending : ListSortDirection.Ascending;

            using (_CoursesView.DeferRefresh())
            {
                _CoursesView.SortDescriptions.Clear();
                _CoursesView.SortDescriptions.Add(new SortDescription("Name", direction));
            }

            _LastSortDirection = _LastSortDirection == ListSortDirection.Ascending ? ListSortDirection.Descending : ListSortDirection.Ascending;
        }

        private void OnCollectionViewCurrentChanged(Object sender, EventArgs e)
        {
            OnPropertyChanged("CurrentCourse");
        }
    }
}
