﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Data;
using ELTE.Data.TravelAgency.RentManager.Model;

namespace ELTE.Data.TravelAgency.RentManager.ViewModel
{
    /// <summary>
    /// Utazási ügynökség nézetmodell típusa.
    /// </summary>
    public class TravelAgencyViewModel : ViewModelBase
    {
        private Operator _Operator;
        private TravelAgencyEntities _Entities;

        /// <summary>
        /// Szűrésszöveg lekérdezése.
        /// </summary>
        public String FilterText { get; private set; }
        /// <summary>
        /// Szűrés engedélyezettségének lekérdezése.
        /// </summary>
        public Boolean FilterEnabled { get { return FilterText == "Szűrés"; } }
        /// <summary>
        /// Státuszüzenet lekérdezése.
        /// </summary>
        public String StatusMessage { get; private set; }

        /// <summary>
        /// Felhasználónév lekérdezése, vagy beállítása.
        /// </summary>
        public String UserName { get; set; }
        /// <summary>
        /// Felhasználó jelszavának lekérdezése, vagy beállítása.
        /// </summary>
        public String UserPassword { get; set; }

        /// <summary>
        /// Foglaló nevének lekérdezése, vagy beállítása.
        /// </summary>
        public String RentName { get; set; }
        /// <summary>
        /// Foglaló címének lekérdezése, vagy beállítása.
        /// </summary>
        public String RentAddress { get; set; }
        /// <summary>
        /// Foglaló telefonszámának lekérdezése, vagy beállítása.
        /// </summary>
        public String RentPhoneNumber { get; set; }
        /// <summary>
        /// Foglaló e-mail címének lekérdezése, vagy beállítása.
        /// </summary>
        public String RentEmail { get; set; }
        /// <summary>
        /// Foglalás kezdődátumának lekérdezése, vagy beállítása.
        /// </summary>
        public DateTime RentStartDate { get; set; }
        /// <summary>
        /// Foglalás befejező dátumának lekérdezése, vagy beállítása.
        /// </summary>
        public DateTime RentEndDate { get; set; }

        /// <summary>
        /// Épületek lekérdezése.
        /// </summary>
        public ObservableCollection<Building> Buildings { get; private set; }
        /// <summary>
        /// Épületek nézetének lekérdezése.
        /// </summary>
        public ICollectionView BuildingsView { get; private set; }

        /// <summary>
        /// Városok lekérdezése.
        /// </summary>
        public ObservableCollection<City> Cities { get; private set; }
        /// <summary>
        /// Aktuális apartman lekérezése.
        /// </summary>
        public Apartment CurrentApartment { get; private set; }
       
        /// <summary>
        /// Bejelentkezés parancs lekérdezése.
        /// </summary>
        public DelegateCommand LoginCommand { get; private set; }
        /// <summary>
        /// Kilépés parancsának lekérdezése.
        /// </summary>
        public DelegateCommand ExitCommand { get; private set; }
        /// <summary>
        /// Szűrés parancs lekérdezése.
        /// </summary>
        public DelegateCommand FilterCommand { get; private set; }
        /// <summary>
        /// Új foglalás parancs lekérdezése.
        /// </summary>
        public DelegateCommand NewRentCommand { get; private set; }
        /// <summary>
        /// Foglalás mentése parancs lekérdezése.
        /// </summary>
        public DelegateCommand SaveRentCommand { get; private set; }
        /// <summary>
        /// Foglalás elvetése parancs lekérdezése.
        /// </summary>
        public DelegateCommand CancelRentCommand { get; private set; }
        /// <summary>
        /// Foglalás törlése parancs lekérdezése.
        /// </summary>
        public DelegateCommand DeleteRentCommand { get; private set; }

        /// <summary>
        /// Bejelentkezés sikertelenségének eseménye.
        /// </summary>
        public event EventHandler LoginFailed;
        /// <summary>
        /// Bejelentkezés sikerességének eseménye.
        /// </summary>
        public event EventHandler LoginSuccess;
        /// <summary>
        /// Új foglalás eseménye.
        /// </summary>
        public event EventHandler NewRent;
        /// <summary>
        /// Foglalás elmentésének eseménye.
        /// </summary>
        public event EventHandler RentSave;
        /// <summary>
        /// Foglalás elvetésének eseménye.
        /// </summary>
        public event EventHandler RentCancel;

        /// <summary>
        /// Utazási ügynökség nézetmodell példányosítása.
        /// </summary>
        public TravelAgencyViewModel()
        {
            _Entities = new TravelAgencyEntities();

            LoginCommand = new DelegateCommand(x => LoginExecuted());
            ExitCommand = new DelegateCommand(x => Application.Current.Shutdown());
            FilterCommand = new DelegateCommand(param => FilterExecuted(param));
            NewRentCommand = new DelegateCommand(param => NewRentExecuted(param));
            SaveRentCommand = new DelegateCommand(x => SaveRentExecuted());
            CancelRentCommand = new DelegateCommand(x => CancelRentExecuted());
            DeleteRentCommand = new DelegateCommand(param => DeleteRentExecuted(param));

            FilterText = "Szűrés";
        }

        /// <summary>
        /// Bejelentkezés.
        /// </summary>
        private void LoginExecuted()
        {
            try
            {
                _Operator = _Entities.Operator.FirstOrDefault(op => op.LoginName == UserName && op.Password == UserPassword);
                // lekérjük a felhasználót a megadott értékek segítségével
                if (_Operator == null) // nem található megfelelő felhasználó
                {
                    OnLoginFailed(); // jelezzük, hogy sikertelen a bejelentkezés
                    return;
                }

                Buildings = new ObservableCollection<Building>(_Entities.Building.Include("Apartment").Include("Apartment.Rent").Include("City"));
                // betöltjük az épületeket, az apartmanokat, az apartmanokhoz tartozó foglalásokat és a városokat is
                BuildingsView = CollectionViewSource.GetDefaultView(Buildings); 
                // szűréshez szükséges nézet példányosítása a gyűjteményre

                // apartmanok betöltése a csatolt táblákkal együtt
                OnPropertyChanged("Buildings");

                Cities = new ObservableCollection<City>(_Entities.City); // városok betöltése
                OnPropertyChanged("Cities");
                OnPropertyChanged("FilterEnabled");

                StatusMessage = "Kész.";
                OnPropertyChanged("StatusMessage");

                OnLoginSuccess(); // jelezzük, hogy sikeres a bejelentkezés
            }
            catch
            {
                OnLoginFailed();
            }
        }

        /// <summary>
        /// Új foglalás.
        /// </summary>
        /// <param name="param">A lefoglalandó apartman.</param>
        private void NewRentExecuted(Object param)
        {
            if (param == null)
            {
                MessageBox.Show("Nincs apartman kiválasztva.", "Sikertelen foglalás", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            if (!(param is Apartment))
            {
                return;
            }

            RentStartDate = DateTime.Now; // aktuális időpont lekérdezése
            RentEndDate = DateTime.Now;
            CurrentApartment = param as Apartment; // elátoljuk a kijelölt apartmant

            OnPropertyChanged("RentStartDate");
            OnPropertyChanged("RentEndDate");
            OnPropertyChanged("CurentApartment");
            OnNewRent();
        }

        /// <summary>
        /// Foglalás mentése.
        /// </summary>
        private void SaveRentExecuted()
        {
            try
            {
                if (CurrentApartment == null)
                    return;

                if (String.IsNullOrEmpty(RentName) || String.IsNullOrEmpty(RentAddress) || String.IsNullOrEmpty(RentPhoneNumber) || String.IsNullOrEmpty(RentEmail))
                {
                    MessageBox.Show("Nincs minden adat megadva.", "Sikertelen foglalás", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }

                // e-mail cím formátumának ellenőrzése
                if (!Regex.IsMatch(RentEmail, @"^([0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*@([0-9a-zA-Z][-\w]*[0-9a-zA-Z]\.)+[a-zA-Z]{2,9})$"))
                {
                    MessageBox.Show("A megadott e-mail cím nem érvényes.", "Sikertelen foglalás", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }

                // ha előbb van a vége, mint a kezdete, vagy a mai napnál korábban kezdődik
                if (RentEndDate <= RentStartDate || RentStartDate <= DateTime.Now)
                {
                    MessageBox.Show("A megadott dátumok nem megfelelőek.", "Sikertelen foglalás", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }

                // kezdőnap ellenőrzése
                if (RentStartDate.DayOfWeek != (DayOfWeek)(CurrentApartment.Turnday - 1))
                {
                    MessageBox.Show("A megadott kezdőnap nem az apartman fordulónapja.", "Sikertelen foglalás", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }

                // befejezés ellenőrzése
                if ((RentEndDate - RentStartDate).TotalDays % 7 != 0)
                {
                    MessageBox.Show("Csak teljes hétre lehet kivenni az aparmtmanokat.", "Sikertelen foglalás", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }

                // időpont-ütközések ellenőrzése
                if (CurrentApartment.Rent.Where(rent => rent.Apartment == CurrentApartment &&
                    (rent.StartDate <= RentStartDate && rent.StartDate + TimeSpan.FromDays(rent.Weeks * 7) > RentStartDate) ||
                    (rent.StartDate <= RentEndDate && rent.StartDate + TimeSpan.FromDays(rent.Weeks * 7) > RentEndDate)).Count() > 0)
                {
                    MessageBox.Show("A megadott időpontra már van foglalás.", "Sikertelen foglalás", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }

                // felvesszünk a foglalást
                _Entities.AddToRent(new Rent
                {
                    ApartmentId = CurrentApartment.Id,
                    Name = RentName,
                    Address = RentAddress,
                    Email = RentEmail,
                    PhoneNumber = RentPhoneNumber,
                    StartDate = RentStartDate.Date,
                    Weeks = (RentEndDate - RentStartDate).Days / 7
                });

                _Entities.SaveChanges(); // mentünk

                OnRentSave();
                StatusMessage = "Foglalás elmentve.";
                OnPropertyChanged("StatusMessage");
            }
            catch
            {
                OnRentCancel();
                StatusMessage = "Foglalás sikertelen.";
                OnPropertyChanged("StatusMessage");
            }
        }

        /// <summary>
        /// Foglalás elvetése.
        /// </summary>
        private void CancelRentExecuted()
        {
            OnRentCancel();
            StatusMessage = "Foglalás elvetve.";
            OnPropertyChanged("StatusMessage");
        }

        /// <summary>
        /// Foglalás törlése.
        /// </summary>
        /// <param name="param">A törlendő foglalás.</param>
        private void DeleteRentExecuted(Object param)
        {
            if (param == null) // ellenőrizzük a paramétert
            {
                MessageBox.Show("Nincs apartman kiválasztva.", "Sikertelen törlés", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            if (!(param is Rent))
                return;

            // ellenőrizzük a törlési szándékot
            if (MessageBox.Show("Biztos törölni szeretnéd a foglalást?", "Törlés megerősítése", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.No)
            {
                return;
            }

            _Entities.DeleteObject(param); // foglalás törlése
            _Entities.SaveChanges();

            StatusMessage = "Foglalás törölve.";
            OnPropertyChanged("StatusMessage");
        }

        /// <summary>
        /// Szűrés végrehajtása.
        /// </summary>
        /// <param name="obj">Szűrő paraméter.</param>
        private void FilterExecuted(Object obj)
        {
            if (FilterText == "Szűrés")
            {
                BuildingsView.Filter = (building => (building as Building).City.Name.Equals(obj)); // szűrő felvétele
                FilterText = "Szűrés kikapcsolása";
                OnPropertyChanged("FilterText");
                OnPropertyChanged("FilterEnabled");
            }
            else
            {
                BuildingsView.Filter = null; // szűrő törlése
                FilterText = "Szűrés";
                OnPropertyChanged("FilterText");
                OnPropertyChanged("FilterEnabled");
            }
        }

        /// <summary>
        /// Bejelentkezés sikertelenségének eseménykiváltása.
        /// </summary>
        private void OnLoginFailed()
        {
            if (LoginFailed != null)
                LoginFailed(this, EventArgs.Empty);
        }
        /// <summary>
        /// Bejelentkezés sikerességének eseménykiváltása.
        /// </summary>
        private void OnLoginSuccess()
        {
            if (LoginSuccess != null)
                LoginSuccess(this, EventArgs.Empty);
        }
        /// <summary>
        /// Új foglalás eseménykiváltása.
        /// </summary>
        private void OnNewRent()
        {
            if (NewRent != null)
                NewRent(this, EventArgs.Empty);
        }
        /// <summary>
        /// Foglalás mentésének eseménykiváltása.
        /// </summary>
        private void OnRentSave()
        {
            if (RentSave != null)
                RentSave(this, EventArgs.Empty);
        }
        /// <summary>
        /// Foglalás elvetésének eseménykiváltása.
        /// </summary>
        private void OnRentCancel()
        {
            if (RentCancel != null)
                RentCancel(this, EventArgs.Empty);
        }
    }
}
